# mongod.conf

# for documentation of all options, see:
#   http://docs.mongodb.org/manual/reference/configuration-options/

# where to write logging data.
systemLog:
  destination: file
  logAppend: true
  path: /var/log/mongodb/mongod.log

# Where and how to store data.
storage:
  dbPath: {{ db_path }}
  journal:
    enabled: true
  engine: "wiredTiger"

# how the process runs
processManagement:
{% if ansible_os_family == "RedHat" %}  # Breaks Ubuntu / Debian
  fork: true
  pidFilePath: /var/run/mongodb/mongod.pid
{% endif %}
  timeZoneInfo: /usr/share/zoneinfo

# network interfaces
net:
  port: {{ config_port }}
  bindIp: {{ bind_ip }}

{% if authorization == "enabled" %}
security:
    authorization: {{ authorization }}
    keyFile: /etc/keyfile
{% endif %}

#operationProfiling:

{% if replicaset %}
replication:
  replSetName: {{ config_repl_set_name }}
  {% endif %}

sharding:
  clusterRole: configsvr
